@@ -1,10 +1,11 @@
 """
 Module for interacting with mzML files
 """
-from .structure import mzML
+from .structure import mzML, Trace
 
 __all__ = [
     'mzML',
+    'Trace',
 ]
 
 
