@@ -5,8 +5,10 @@ import gzip
 import sys
 import warnings
 from random import random
-from typing import Generator
+from typing import Generator, List, MutableMapping
 from xml.etree import ElementTree
+from functools import wraps
+from collections.abc import MutableSequence
 
 import scipy as sci
 from tqdm import tqdm
@@ -45,6 +47,110 @@ class BoundsError(Warning):
             self.warned[name] += 1
 
 
+class Trace(MutableSequence):
+    def __init__(self,
+                 *incoming_values: float,
+                 center_bin: bool = False,
+                 ):
+        """
+        A data class for managing binned values of a trace. The class stores raw data and retains binned traces
+        after they are generated (prevents re-binning data when the data is used in multiple locations).
+
+        :param incoming_values: values to add on instantiation
+        :param center_bin: whether to center the bin value
+        """
+        self._raw_data: List[float] = []
+        self._binned_data: MutableMapping[int, List[float]] = {}
+        self.center_bin: bool = center_bin
+        # self._bin_valid: bool = False  # todo enable tracking of bin validity (once new value added, prev bins invalid)
+
+        for value in incoming_values:
+            self.append(value)
+
+    def __getitem__(self, item) -> float:
+        return self._raw_data[item]
+
+    def __setitem__(self, key, value):
+        self._raw_data[key] = value
+
+    def __delitem__(self, key):
+        del self._raw_data[key]
+
+    def __len__(self):
+        return len(self._raw_data)
+
+    def __copy__(self):
+        return self.__class__(
+            *self._raw_data,
+            center_bin=self.center_bin,
+        )
+
+    @property
+    def raw_data(self) -> List[float]:
+        """raw data list (unbinned)"""
+        return self._raw_data
+
+    def insert(self, index: int, val: float) -> None:
+        """
+        Inserts a value at the position specified
+
+        :param index: index
+        :param val: value
+        """
+        self._raw_data.append(val)
+
+    def get_binned_data(self, bin_num: int) -> List[float]:
+        """
+        Bins the data into bins of the size specified. If the data has already been binned to that level, returns the
+        previously binned data.
+
+        :param bin_num: bin number to use
+        :return: list of binned values
+        """
+        # if previously binned, return
+        if bin_num in self._binned_data:
+            return self._binned_data[bin_num]
+        # bin data
+        out = self.bin_trace(
+            bin_num,
+            self.raw_data,
+            bin_denom=bin_num if self.center_bin else 1,
+        )
+        # store data for later retrieval
+        self._binned_data[bin_num] = out
+        return out
+
+    @staticmethod
+    def bin_trace(bin_num: int, target_list: List[float], bin_denom=1):
+        """
+        Bins a list of values into bins of size *bin_num*.
+
+        :param bin_num: Number of values to bin together. e.g. ``n = 4`` would bin the first four values into a single
+            value, then the next 4, etc.
+        :param target_list: List of values to bin.
+        :param bin_denom: Bin denominator. The calculated bin values will be divided by this value. e.g. if ``n = v`` the
+            output values will be an average of each bin.
+        :return: binned list
+
+        **Notes**
+
+        - If the size of the list is not divisible by `bin_num`, the final bin will not be included in the output list.
+        (The last values will be discarded.)
+
+        """
+        out = []
+        delta = 0
+        ttemp = 0
+        for ind, val in enumerate(target_list):
+            delta += 1
+            ttemp += val  # add current value to growing sum
+            if delta == bin_num:  # critical number is reached
+                out.append(ttemp / float(bin_denom))  # append sum to list
+                delta = 0  # reset critical count and sum
+                ttemp = 0
+        return out
+
+
 class mzML(object):
     def __init__(self,
                  filename: str,
@@ -124,7 +230,7 @@ class mzML(object):
             self.nchroms = 0
 
         self.functions = {}
-        for spectrum in self._spectra_elements:
+        for spectrum in self.spectra_elements:
             try:
                 # try to retrieve function, process, and scan from attributes
                 func, proc, scan = fps(spectrum)  # extract each value and convert to integer
@@ -150,9 +256,9 @@ class mzML(object):
             self.duration = None
 
         self._BE = BoundsError()  # load warning instance for integration
-        self.ftt = False
+        self._ftt_extracted = False
         if ftt is True:
-            self.function_timetic()
+            self.extract_function_time_tic()
 
     def __str__(self):
         """The string that is returned when printed"""
@@ -187,7 +293,7 @@ class mzML(object):
             if ind < 0 or ind > self.nscans:
                 raise IndexError("The scan index number #%d is outside of the mzML's scan index range (0-%d)" % (
                 ind, self.nscans - 1))
-            for spectrum in self._spectra_elements:
+            for spectrum in self.spectra_elements:
                 attr = branch_attributes(spectrum)
                 if attr['index'] == ind:
                     return spectrum_array(spectrum)
@@ -198,19 +304,19 @@ class mzML(object):
                 raise ValueError(
                     "The supplied time %.3f is outside of this file's time range (0 - %.3f)" % (ind, self.duration))
             ind = self.scan_index(ind)
-            for spectrum in self._spectra_elements:
+            for spectrum in self.spectra_elements:
                 attr = branch_attributes(spectrum)
                 if attr['index'] == ind:
                     return spectrum_array(spectrum)
 
     @property
-    def _spectra_elements(self) -> Generator[ElementTree.Element, None, None]:
+    def spectra_elements(self) -> Generator[ElementTree.Element, None, None]:
         """generator of spectra elements"""
         for element in _findall_ele('mzML', 'run', 'spectrumList', 'spectrum', parent=self.root):
             yield element
 
     @property
-    def _chromatogram_elements(self) -> Generator[ElementTree.Element, None, None]:
+    def chromatogram_elements(self) -> Generator[ElementTree.Element, None, None]:
         """generator of chromatogram elements"""
         for element in _findall_ele('mzML', 'run', 'chromatogramList', 'chromatogram', parent=self.root):
             yield element
@@ -243,7 +349,7 @@ class mzML(object):
             """decorates the supplied function to run for every scan"""
             out = []
             msg = f'applying function to chromatogram'
-            for chromatogram in tqdm(self._chromatogram_elements, desc=msg, disable=self._mute_tqdm):
+            for chromatogram in tqdm(self.chromatogram_elements, desc=msg, disable=self._mute_tqdm):
                 out.append(fn(chromatogram, *args, **kwargs))
             return out
 
@@ -274,7 +380,7 @@ class mzML(object):
             """decorates the supplied function to run for every scan"""
             out = []
             msg = 'applying function to spectrum'
-            for spectrum in tqdm(self._spectra_elements, desc=msg, disable=self._mute_tqdm):
+            for spectrum in tqdm(self.spectra_elements, desc=msg, disable=self._mute_tqdm):
                 out.append(fn(spectrum, *args, **kwargs))
             return out
 
@@ -324,7 +430,7 @@ class mzML(object):
             if 'level' in dct:
                 level = dct['level']
 
-        if affin is None and level is None:
+        if affin is None and level in [None, 1]:
             return min(self.functions.keys())  # assume first function
 
         elif affin == 'UV':  # if UV-Vis affinity
@@ -455,26 +561,60 @@ class mzML(object):
                 return pw_convert(fn, self.precision, self.compression, self.gzip_file, verbose=self.verbose)
             return fn
 
-    def function_timetic(self):
+    def _ensure_ftt_extracted(fn):
+        """
+        checks whether function, time, and TIC lists have been extracted and extracts them if not
+        (runtime method of ensuring the data is there)
+        """
+        @wraps(fn)
+        def decorated(self: 'mzML', *args, **kwargs):
+            if self._ftt_extracted is False:
+                self.extract_function_time_tic()
+            return fn(self, *args, **kwargs)
+        return decorated
+
+    def extract_function_time_tic(self):
         """
         extracts timepoints and tic lists for each function
         this function is separate from mzml contents because it would increase load times significantly (~6x)
         """
+        # todo see if there's a way to sneakily do this the first time the user iterates through spectra
+        # todo come up with a data class to manage function metadata
         msg = 'extracting timepoints and total ion current values'
         for function in self.functions:  # add timepoint and tic lists
-            self.functions[function]['timepoints'] = []  # list for timepoints
-            self.functions[function]['tic'] = []  # list for total ion current values
+            self.functions[function]['timepoints'] = Trace(center_bin=True)  # list for timepoints
+            self.functions[function]['tic'] = Trace()  # list for total ion current values
             if 'level' in self.functions[function] and self.functions[function]['level'] > 1:
-                self.functions[function]['ce'] = []  # list for collision energies
-        for spectrum in tqdm(self._spectra_elements, desc=msg, disable=self._mute_tqdm):
-            attr = branch_attributes(spectrum)
+                self.functions[function]['ce'] = Trace()  # list for collision energies
+        for spectrum in tqdm(self.spectra_elements, desc=msg, disable=self._mute_tqdm, total=self.nscans):
+            # attr = branch_attributes(spectrum)
             function, proc, scan = fps(spectrum)  # determine function, process, and scan numbers
             p = CVParameterSet.create_from_branch(spectrum)  # pull spectrum's cvparameters
             self.functions[function]['timepoints'].append(p['MS:1000016'].value)  # start scan time
             self.functions[function]['tic'].append(p['MS:1000285'].value)  # total ion current
             if 'MS:1000045' in p:
                 self.functions[function]['ce'].append(p['MS:1000045'].value)  # collision energy
-        self.ftt = True
+        self._ftt_extracted = True
+
+    @_ensure_ftt_extracted
+    def get_timepoints_of_function(self, function: int) -> Trace:
+        """
+        Retrieves the timepoints of the specified function
+
+        :param function: function number
+        :return: Trace of data
+        """
+        return self.functions[function]['timepoints']
+
+    @_ensure_ftt_extracted
+    def get_tic_of_function(self, function: int) -> Trace:
+        """
+        Retrieves the total ion current (TIC) of the specified function
+
+        :param function: function number
+        :return: Trace of data
+        """
+        return self.functions[function]['tic']
 
     def integrate(self, name, start, end, x, y):
         """
@@ -523,7 +663,7 @@ class mzML(object):
         """
         msg = 'extracting chromatogram'
         chroms = {}  # dictionary of chromatograms
-        for chromatogram in tqdm(self._chromatogram_elements, desc=msg, disable=self._mute_tqdm):
+        for chromatogram in tqdm(self.chromatogram_elements, desc=msg, disable=self._mute_tqdm):
             attr = branch_attributes(chromatogram)  # pull attributes
             chrom_array = spectrum_array(chromatogram)
             xunit, yunit = get_element_units(chromatogram)
@@ -535,6 +675,7 @@ class mzML(object):
             }
         return chroms
 
+    @_ensure_ftt_extracted
     def pull_species_data(self, sp, sumspec=False):
         """
         Extracts integrated data at every timepoint for all species specified in the sp dictionary
@@ -578,11 +719,9 @@ class mzML(object):
                 dct=sp[species])  # associate each species in the spectrum with a function
             if 'raw' not in sp[species]:  # look for empty raw list
                 sp[species]['raw'] = []
-        if self.ftt is False:  # if timepoints and tic values have not been extracted yet, extract those
-            self.function_timetic()
 
         msg = 'extracting species data from spectrum'
-        for spectrum in tqdm(self._spectra_elements, desc=msg, disable=self._mute_tqdm):
+        for spectrum in tqdm(self.spectra_elements, desc=msg, disable=self._mute_tqdm, total=self.nscans):
             function, proc, scan = fps(spectrum)  # pull function, process, and scan numbers
             # attr = branch_attributes(spectrum)  # get attributes
             spec_array = spectrum_array(spectrum)  # generate spectrum
@@ -610,6 +749,7 @@ class mzML(object):
         """a generator for scans in """
         return
 
+    @_ensure_ftt_extracted
     def retrieve_scans(self, start=None, end=None, mzstart=None, mzend=None, function=None, mute=False, outside=False):
         """
         Retrieves the specified scans or time range from the specified function
@@ -643,11 +783,9 @@ class mzML(object):
             raise ValueError('The function "%d" is not in this mzml file.' % function)
         start = self.scan_index(start, function, bias='greater')
         end = self.scan_index(end, function, bias='lesser')
-        if self.ftt is False:  # extract the timepoints and etc from the mzml
-            self.function_timetic()
         msg = 'retrieving scans'
         out = []
-        for spectrum in tqdm(self._spectra_elements, desc=msg, disable=self._mute_tqdm, total=end-start):
+        for spectrum in tqdm(self.spectra_elements, desc=msg, disable=self._mute_tqdm, total=end - start):
             attr = branch_attributes(spectrum)
             # func,proc,scan = self.fps(spectrum) # determine function, process, and scan numbers
             # p = CVParameterSet.create_from_branch(spectrum)
@@ -668,6 +806,7 @@ class mzML(object):
             return out[0]
         return out
 
+    @_ensure_ftt_extracted
     def scan_index(self, scan=None, function=1, bias='lesser'):
         """
         Determines the index for a scan or timepoint in a given function
@@ -686,8 +825,6 @@ class mzML(object):
             if bias == 'lesser':  # used for end point
                 return self.functions[function]['sr'][1]
         if type(scan) is float:  # timepoint
-            if self.ftt is False:
-                self.function_timetic()
             # return located index plus start of the scan range
             return locate_in_list(self.functions[function]['timepoints'], scan, bias=bias) + self.functions[function]['sr'][0]
         elif type(scan) is int:  # scan number
@@ -745,7 +882,7 @@ class mzML(object):
         )
 
         msg = 'combining spectra'
-        for spectrum in tqdm(self._spectra_elements, desc=msg, disable=self._mute_tqdm or mute, total=end-start):
+        for spectrum in tqdm(self.spectra_elements, desc=msg, disable=self._mute_tqdm or mute, total=end - start):
             attr = branch_attributes(spectrum)  # get attributes
             if attr['index'] > end:
                 break
@@ -754,3 +891,6 @@ class mzML(object):
                 spec.add_spectrum(x, y)  # add spectrum to Spectrum object
         out = spec.trim()
         return out
+
+    # set as static method for decorator to work
+    _ensure_ftt_extracted = staticmethod(_ensure_ftt_extracted)
