@@ -44,6 +44,7 @@ import sys
 import warnings
 import scipy as sci
 import numpy as np
+from typing import Iterable
 from .spectrum import Spectrum
 from bisect import bisect_left, bisect_right
 from .colour import Colour
@@ -497,6 +498,29 @@ def normalize(lst, maxval=1.):
     return lst
 
 
+def normalize_lists(target: Iterable[float], divisor: Iterable[float]) -> Iterable:
+    """
+    Normalizes a target list value-by-value by the divisor list. Lists must be the same size
+
+    :param target: target list
+    :param divisor: list to divide each element of the target list by
+    :return: list of normalized target values
+    """
+    if len(target) != len(divisor):
+        raise ValueError(
+            f'the length of the target ({len(target)}) is not equal to the length of the divisor list ({len(divisor)}'
+        )
+    out = []
+    for num, denom in zip(target, divisor):
+        try:
+            out.append(num / denom)
+        except (ValueError, ZeroDivisionError):
+            # original method of avoiding zero division error
+            # todo consider appending None instead
+            out.append(num / 0.0001)
+    return out
+
+
 def localmax(x: list, y: list, xval: float, lookwithin: float = 1.):
     """
     Finds the local maximum within +/- lookwithin of the xval
