@@ -3,6 +3,8 @@ Class for opening and handling excel files with commonly used data formats
 """
 import sys
 import openpyxl as op
+import numpy as np
+from typing import Iterable
 from openpyxl.utils.exceptions import InvalidFileException
 
 
@@ -1005,6 +1007,30 @@ class XLSX(object):
             chart.series.append(series)  # add the data to the chart
             ws.add_chart(chart, 'E1')  # add the chart to the worksheet
 
+    def write_array_to_sheet(self,
+                             arr: np.ndarray,
+                             header_row: Iterable = None,
+                             sheetname: str = None,
+                             save: bool = False,
+                             ):
+        """
+        Writes the provided data array to sheet.
+
+        :param arr: numpy ndarray. Expects the first dimension of the array to be row-like.
+            (the function will iterate over the first dimension and write number of columns equivalent to the
+            size of the second dimension.
+        :param sheetname: sheet to write the data to
+        :param save: whether to save after finishing
+        """
+        sheet = self.wb.create_sheet(sheetname)
+        # if a header row was provided
+        if header_row is not None:
+            sheet.append(header_row)
+        for row in arr:
+            sheet.append(list(row))
+        if save is True:
+            self.save()
+
 
 if __name__ == '__main__':
     name = 'Useless delete this'
